import refy
from pathlib import Path
import pytest


@pytest.fixture
def csv_path():
    htmlpath = Path("test.csv")
    if htmlpath.exists():
        htmlpath.unlink()
    return htmlpath


def test_by_author(csv_path):
    refy.query_author("Gary  Stacey", csv_path=csv_path)

    refy.query_author("Gary  Stacey", to=1990, since=2020)

    # refy.query_author("this author doesnt exist probably right")


def test_query(csv_path):
    refy.query("neuron gene expression", N=20, csv_path=csv_path)

    # refy.query(
    #     "neuron gene expression", N=20, since=2015, to=2018,
    # )

    # refy.query(
    #     " i dont think youll find many recomendations with this my dear 1231231"
    # )


# def test_query_save():
#     f = refy.base_dir / "query.csv"
#     if f.exists:
#         f.unlink()

#     refy.query("neuron gene expression", N=20, savepath=f)

#     assert f.exists, "should have saved"
#     f.unlink()



from refy import utils
from refy import base_dir


def test_isin():
    l1 = [1, 2, 3]
    l2 = [3, 4, 5]
    l3 = [5, 6, 7]

    assert utils.isin(l1, l2), "should be true"
    assert not utils.isin(l1, l3), "shoule be false"


def test_json():
    path = base_dir / "test.json"
    if path.exists():
        path.unlink()

    a = {str(i): i for i in range(20)}

    utils.to_json(a, path)
    assert path.exists()

    b = utils.from_json(path)
    assert a == b, "should be the same"

    path.unlink()


def test_request():
    if not utils.check_internet_connection():
        return

    url = "https://api.biorxiv.org/details/biorxiv/2015-01-01/2015-01-02"
    utils.request(url)



import multiprocessing
from loguru import logger
import requests
import sys
from concurrent.futures import ThreadPoolExecutor

sys.path.append("./")

from refy.settings import (
    database_path,
    abstracts_path,
    remote_url_base,
    example_path,
    d2v_model_path,
    biorxiv_abstracts_path,
    biorxiv_database_path,
    test_database_path,
    test_abstracts_path,
)

from refy.utils import _request, check_internet_connection
from refy.progress import http_retrieve_progress


# get urls
database_url = remote_url_base + "database.ftr"
abstracts_url = remote_url_base + "abstracts.json"
test_database_url = remote_url_base + "test_database.ftr"
test_abstracts_url = remote_url_base + "test_abstracts.json"
biorxiv_database_url = remote_url_base + "biorxiv_database.ftr"
biorxiv_abstracts_url = remote_url_base + "biorxiv_abstracts.json"
d2v_model = remote_url_base + "d2v_model.model"
d2v_vecs = remote_url_base + "d2v_model.model.docvecs.vectors_docs.npy"
d2v_wv = remote_url_base + "d2v_model.model.wv.vectors.npy"
d2v_sin = remote_url_base + "d2v_model.model.trainables.syn1neg.npy"
example_library = remote_url_base + "example_library.bib"

# organize urls and paths
d2v_base = d2v_model_path.parent
data = [
    (example_library, example_path),
    (database_url, database_path),
    (abstracts_url, abstracts_path),
    (test_database_url, test_database_path),
    (test_abstracts_url, test_abstracts_path),
    (biorxiv_database_url, biorxiv_database_path),
    (biorxiv_abstracts_url, biorxiv_abstracts_path),
    (d2v_model, d2v_model_path),
    (d2v_vecs, d2v_base / "d2v_model.model.docvecs.vectors_docs.npy"),
    (d2v_wv, d2v_base / "d2v_model.model.wv.vectors.npy"),
    (d2v_sin, d2v_base / "d2v_model.model.trainables.syn1neg.npy"),
]


def retrieve_over_http(url, response, output_file_path, task_id):
    """
        Download file from remote location, with progress bar.

        Arguments: 
            url: srt. Remote url to download the data from
            response: response object from sending a request to a target url
            output_file_path: Path. Path to where the downloaded data will be stored
            task_id: task id to update multi-line progress bar
    """

    CHUNK_SIZE = 4096

    try:
        with open(output_file_path, "wb") as fout:
            for chunk in response.iter_content(chunk_size=CHUNK_SIZE):
                fout.write(chunk)
                http_retrieve_progress.update(task_id, advance=len(chunk))

    except requests.exceptions.ConnectionError:
        output_file_path.unlink()
        raise requests.exceptions.ConnectionError(
            f"Could not download file from {url}"
        )

    http_retrieve_progress.remove_task(task_id)
    logger.debug(f"Done downloading {output_file_path.name}")


def download_all():
    """
        Download all data used by refy, in parallel for speed.
        It checks if all files used by refy are present and if not
        it downloads them from GIN: 
            https://gin.g-node.org/FedeClaudi/refy/src/master/
    """
    logger.debug("Checking that all files are present")

    # download in parallel
    n_cpus = multiprocessing.cpu_count() - 2
    with http_retrieve_progress as progress:
        with ThreadPoolExecutor(max_workers=n_cpus) as pool:
            for (url, output_file_path) in data:

                # check if file was downloaded already
                if output_file_path.exists():
                    logger.debug(
                        f"Not downloading {output_file_path.name} because it exists already"
                    )
                    continue

                # send a request and start a progress bar
                response = _request(url, stream=True,)

                task_id = progress.add_task(
                    "download",
                    start=True,
                    total=int(response.headers.get("content-length", 0)),
                    filename=output_file_path.name,
                )

                # stream data to file
                pool.submit(
                    retrieve_over_http,
                    url,
                    response,
                    output_file_path,
                    task_id,
                )


def check_all():
    """
        Checks that all necessary files are present
    """
    for url, filepath in data:
        if not filepath.exists():
            return False, filepath
        else:
            return True, None


def check_files():
    """
            Checks that all necessary files are present and tries to download them if not
        """
    # download all missing if connection
    if check_internet_connection():
        download_all()

    # check all files are there
    passed, missing = check_all()
    if not passed:
        raise ValueError(
            f"At least one necessary file missing: {missing.name}. Connect to the internet to download"
        )


if __name__ == "__main__":
    download_all()



import os
from pathlib import Path

DEBUG = False
TEST_MODE = False  # gets set to True when running tests

# ----------------------------------- paths ---------------------------------- #
# create base path folder
base_dir = Path(os.path.join(os.path.expanduser("~"), ".refy"))
base_dir.mkdir(exist_ok=True)

# semantic scholar dbase paths
abstracts_path = base_dir / "abstracts.json"
database_path = base_dir / "database.ftr"

# test dbase paths
test_abstracts_path = base_dir / "test_abstracts.json"
test_database_path = base_dir / "test_database.ftr"

# biorxiv dbase paths
biorxiv_abstracts_path = base_dir / "biorxiv_abstracts.json"
biorxiv_database_path = base_dir / "biorxiv_database.ftr"

# d2v model paths
d2v_model_path = base_dir / "d2v_model.model"

# example library
example_path = base_dir / "example_library.bib"

# urls
remote_url_base = "https://gin.g-node.org/FedeClaudi/refy/raw/master/"

# ----------------------------- database settings ---------------------------- #
# when creating condensed database, keep only papers in these fields
fields_of_study = (
    "biology",
    "neuroscience",
    "medicine",
    "machine learning",
    "deep learning",
    "robotics",
)
keywords = (
    "neuro",
    "neuron",
    "brain",
    "synapse",
    "neurons",
    "neurotransmitter",
    "neuronal",
    "behaviour",
    "behavior",
    "cognition",
    "neural network",
    "deep learning",
    "dendrite",
    "dendritic",
    "behaviour",
    "robotics",
    "artificial agent",
    "reinforcement learning",
    "agent",
    "computational",
    "dynamics",
)  # only keep papers that have these keywords in the abstract


low_year = 1990  # only papers more recent than this are kept


# start and end date for queries from biorxiv's server
biorxiv_start_date = "2015-01-01"
biorxiv_end_date = "2021-01-15"



from refy import settings
import refy

# set test mode to load a small portion of the database
settings.TEST_MODE = True
settings.DEBUG = True
refy.set_logging("DEBUG")


DB = refy.database.load_database()



from refy import suggest
from refy.settings import example_path, base_dir
import pandas as pd


def test_suggest_save():
    # create a path to save the suggestions to
    save_path = base_dir / "ref_test.csv"
    if save_path.exists():
        save_path.unlink()

    # get suggestions
    suggestions = suggest(example_path, N=20, csv_path=save_path).suggestions

    # check suggestions
    assert len(suggestions) == 20, "wrong number of sugg"

    # check saved suggestions
    assert save_path.exists(), "didnt save"
    saved = pd.read_csv(save_path)

    assert len(saved) == 20, "loaded has wrong length"

    save_path.unlink()


def test_suggest_criteria():

    # test criterias
    suggestions = suggest(example_path, N=20, since=2015, to=2019).suggestions

    assert suggestions.suggestions.year.min() == 2015, "since doesnt work"
    assert suggestions.suggestions.year.max() == 2019, "to doesnt work"



from datetime import datetime, timedelta
from loguru import logger
import math
import pandas as pd
from numpy import dot
from numpy.linalg import norm
from myterial import orange, green
from pathlib import Path
import sys
from crontab import CronTab


sys.path.append("./")

from refy.utils import check_internet_connection, request, open_in_browser
from refy.settings import fields_of_study, base_dir
from refy.input import load_user_input
from refy._query import SimpleQuery

from refy.doc2vec import D2V

base_url = "https://api.biorxiv.org/details/biorxiv/"


def cosine(v1, v2):
    """
        Cosine similarity between two vectors
    """
    return dot(v1, v2) / (norm(v1) * norm(v2))


class Daily(SimpleQuery):
    def __init__(
        self, user_data_filepath, html_path=None, N=10, show_html=True
    ):
        """
            Get biorxiv preprints released in the last 24 hours
            and select the top N matches based on user inputs
            
            Arguments:
                user_data_filepath: str, Path. Path to user's .bib fole
                html_path: str, Path. Path to a .html file 
                    where to save the output
                N: int. Number of papers to return
                show_html: bool. If true and a html_path is passed, it opens
                    the html in the default web browser
        """
        logger.debug("\n\nStarting biorxiv daily search")
        self.start(text="Getting daily suggestions")

        SimpleQuery.__init__(self, html_path=html_path)
        self.model = D2V()

        # get data from biorxiv
        logger.debug("Getting data from biorxiv")
        self.fetch()

        # load user data
        logger.debug("Loading user papers")
        self.user_papers = load_user_input(user_data_filepath)

        # embed biorxiv's papers
        logger.debug("Embedding papers")
        self.papers_vecs = {
            ID: self.model._infer(abstract)
            for ID, abstract in self.abstracts.items()
        }

        # embed user data
        self.user_papers_vecs = {
            p["id"]: self.model._infer(p.abstract)
            for i, p in self.user_papers.iterrows()
        }

        # get suggestions
        logger.debug("Retuning suggestions")
        self.get_suggestions(N)

        # get keyords
        logger.debug("Getting keywords")
        self.get_keywords(self.user_papers)
        self.stop()

        # print
        today = datetime.today().strftime("%Y-%m-%d")
        self.print(
            text=f"[{orange}]:calendar:  Daily suggestions for: [{green} bold]{today}\n\n"
        )

        # save to html
        self.to_html(
            text=f"[{orange}]:calendar:  Daily suggestions for: [{green} bold]{today}\n\n",
        )

        # open html in browser
        if self.html_path is not None and show_html:
            open_in_browser(self.html_path)

    def clean(self, papers):
        """
            Cleans up a set of papers

            Arguments:
                papers: pd.DataFrame

            Return:
                papers: cleaned up papers
                abstracts: dict of papers abstracts
        """
        # keep only relevant papers/info
        papers = pd.DataFrame(papers)
        if papers.empty:
            raise ValueError("No papers were downloaded from biorxiv")

        papers = papers[
            ["doi", "title", "authors", "date", "category", "abstract"]
        ]
        papers = papers.loc[papers.category.isin(fields_of_study)]

        # fix ID
        papers["id"] = papers["doi"]
        papers = papers.drop_duplicates(subset="id")

        # fix year of publication
        papers["year"] = [p.date.split("-")[0] for i, p in papers.iterrows()]
        del papers["date"]

        # separate abstracts
        abstracts = {
            paper.id: paper.abstract for i, paper in papers.iterrows()
        }
        del papers["abstract"]

        # make sure everything checks out
        papers = papers.loc[papers["id"].isin(abstracts.keys())]
        papers = papers.drop_duplicates(subset="id")
        papers["source"] = "biorxiv"

        return papers, abstracts

    def fetch(self):
        """
            Downloads latest biorxiv's preprints, hot off the press
        """
        if not check_internet_connection():
            raise ConnectionError(
                "Internet connection needed to download data from biorxiv"
            )

        today = datetime.today().strftime("%Y-%m-%d")
        yesterday = (datetime.now() - timedelta(1)).strftime("%Y-%m-%d")

        req = request(base_url + f"{yesterday}/{today}")
        tot = req["messages"][0]["total"]
        logger.debug(
            f"Downloading metadata for {tot} papers || {yesterday} -> {today}"
        )

        # loop over all papers
        papers, cursor = [], 0
        while cursor < int(math.ceil(tot / 100.0)) * 100:
            # download
            papers.append(
                request(base_url + f"{yesterday}/{today}/{cursor}")[
                    "collection"
                ]
            )
            cursor += 100

        # clean up and get abstracts
        papers = pd.concat([pd.DataFrame(ppr) for ppr in papers])
        self.papers, self.abstracts = self.clean(papers)
        logger.debug(f"Kept {len(self.papers)} biorxiv papers")

    def get_suggestions(self, N):
        """
            Computes the average cosine similarity
            between the input user papers and those from biorxiv, 
            then uses the distance to sort the biorxiv papers
            and select the best 10

            Arguments:
                N: int. number of papers to suggest
        """
        logger.debug("getting suggestions")

        # compute cosine distances
        distances = {ID: 0 for ID in self.papers_vecs.keys()}
        for uID, uvec in self.user_papers_vecs.items():
            for ID, vector in self.papers_vecs.items():
                distances[ID] += cosine(uvec, vector)

        distances = {ID: d / len(self.papers) for ID, d in distances.items()}

        # sort and truncate
        self.fill(self.papers, len(distances), None, None, ignore_authors=True)
        self.suggestions.set_score(distances.values())
        self.suggestions.truncate(N)


def setup(user, python_path, bibfile, N, outputpath):
    """
        Sets up crontab schedule to run Daily every day

        Arguments:
            user: str. user name
            python_path: str. path to python installation to use
            N: int. Number of suggestions to return
            outputpath: str. Path to .HTML to save the results to
    """
    if not Path(python_path).exists():
        raise FileNotFoundError(
            f"Could not find .bib file to use for daily searches: {python_path}"
        )

    # get refy installation path
    refy_folder = Path(__file__).parent
    assert (refy_folder / "cli.py").exists(), "could not find refy's cli file"
    logger.debug(f"Refy package folder: {refy_folder}")

    # get out file
    out_file = base_dir / "daily.txt"
    if out_file.exists():
        out_file.unlink()

    # create crontab command
    command = f"{python_path} {refy_folder}/cli.py daily {bibfile} -N {N} -o {outputpath} -show-html false --d"
    command += f" >> {out_file}"  # output file for crontab

    # setup cronotab job
    logger.debug(f"Setting up crontab for user: {user}")
    cron = CronTab(user=user)

    logger.debug(f"setting up crontab jobs with command:\n     {command}")
    job = cron.new(command=command, comment="refy_daily")
    job.setall("* 7 * * *")  # run at 7 AM every day
    cron.write()

    jobs = "\n".join([str(c) for c in cron])
    logger.debug(f"Crontab jobs:\n{jobs}")


def stop(user):
    """
        Removes crontab job for refy
    """
    cron = CronTab(user=user)
    cron.remove_all(comment="refy_daily")
    cron.write()

    # log jobs
    jobs = "\n".join([str(c) for c in cron])
    logger.debug(f"Crontab jobs:\n{jobs}")


if __name__ == "__main__":
    import refy

    refy.set_logging("DEBUG")
    d = Daily(refy.settings.example_path, html_path="test.html")

    # setup(
    #     "federico claudi",
    #     "/Users/federicoclaudi/miniconda3/envs/ref/bin/python",
    #     "test.bib",
    # )
    # stop("federico claudi")

    """
        example:

        sudo refy setup-daily "federico claudi" "/Users/federicoclaudi/miniconda3/envs/ref/bin/python" "/Users/federicoclaudi/Documents/Github/referee/test.bib" -N 20 -o /Users/federicoclaudi/Desktop/refy.html

        to stop:

        sudo refy stop-daily "federico claudi"

    """



import typer
import sys
from loguru import logger

sys.path.append("./")
import refy

app = typer.Typer()


@app.command()
def daily(
    filepath: str = typer.Argument(
        ..., help="Path to .bib file with user papers metadata"
    ),
    N: int = typer.Option(10, "-N", help="number of recomendations to return"),
    html_path: str = typer.Option(
        None, "-o", help="Path to .html file where to save suggestions"
    ),
    show_html: bool = typer.Option(
        True, "-show-html", "--s", help="Show HTML output in browser"
    ),
    debug: bool = typer.Option(
        False, "-debug", "--d", help="set debug mode ON/OFF"
    ),
):
    """
        Run suggestions on latest biorxvis preprints, daily.
    """

    # set specific log file
    logpath = refy.base_dir / "daily.log"
    lvl = "DEBUG" if debug else "INFO"
    try:
        refy.set_logging(level=lvl, path=logpath)
    except:
        refy.set_logging(level=lvl)

    # run daily search
    try:
        refy.Daily(filepath, html_path=html_path, N=N, show_html=show_html)
    except Exception as e:
        logger.warning(f"Failed to run daily search, error: {e}")


@app.command()
def setup_daily(
    user: str = typer.Argument(..., help="Computer user name"),
    python_path: str = typer.Argument(
        ..., help="Path of python executable (which python)"
    ),
    filepath: str = typer.Argument(
        ..., help="Path to .bib file with user papers metadata"
    ),
    N: int = typer.Option(10, "-N", help="number of recomendations to return"),
    html_path: str = typer.Option(
        None, "-o", help="Path to .html file where to save suggestions"
    ),
):
    """
        Sets up a scheduled daily task to recomend papers
    """
    # set specific log file
    logpath = refy.base_dir / "daily.log"
    if logpath.exists():
        logpath.unlink()

    refy.set_logging(level="INFO", path=str(logpath))

    # setup daily
    try:
        refy.daily.setup(user, python_path, filepath, N, html_path)
    except OSError:
        raise ValueError(
            "In order to set up refy daily you need to use this command as administrator."
            ' On Linux and Mac OS try putting "sudo" before the command'
        )


@app.command()
def stop_daily(user: str = typer.Argument(..., help="Computer user name"),):
    """
        removes any schedule daily task from refy
    """
    refy.daily.stop(user)


@app.command()
def suggest(
    filepath: str = typer.Argument(
        ..., help="Path to .bib file with user papers metadata"
    ),
    N: int = typer.Option(25, "-N", help="number of recomendations to return"),
    since: int = typer.Option(
        None, "-since", help="Only keep papers published after SINCE"
    ),
    to: int = typer.Option(
        None, "-to", help="Only keep papers published before TO"
    ),
    csv_path: str = typer.Option(
        None, "-csv-path", help="Save suggestions to .CSV file"
    ),
    html_path: str = typer.Option(
        None, "-html-path", help="Save suggestions to .HTML file"
    ),
    debug: bool = typer.Option(
        False, "-debug", "--d", help="set debug mode ON/OFF"
    ),
):
    """
        Suggest new relevant papers based on your library

        Arguments:
            user_papers: str, path. Path to a .bib file with user's papers info
            N: int. Number of papers to suggest
            since: int or None. If an int is passed it must be a year,
                only papers more recent than the given year are kept for recomendation
            to: int or None. If an int is passed it must be a year,
                only papers older than that are kept for recomendation
            csv_path: str, Path. Path pointing to a .csv file where the recomendations
                will be saved
            html_path: str, Path. Path to save formatted suggestions as HTML content
            debug: bool. If true refy is set in debug mode and more info are printed
    """
    if debug:
        refy.set_logging("DEBUG")

    logger.debug(f"CLI: suggest for: {filepath}")

    refy.suggest(
        filepath,
        N=N,
        since=since,
        to=to,
        csv_path=csv_path,
        html_path=html_path,
    )


@app.command()
def query(
    input_string: str = typer.Argument(..., help="Imput string for query"),
    N: int = typer.Option(10, "-N", help="number of recomendations to return"),
    since: int = typer.Option(
        None, "-since", help="Only keep papers published after SINCE"
    ),
    to: int = typer.Option(
        None, "-to", help="Only keep papers published before TO"
    ),
    csv_path: str = typer.Option(
        None, "-csv-path", help="Save suggestions to file"
    ),
    debug: bool = typer.Option(
        False, "-debug", "--d", help="set debug mode ON/OFF"
    ),
):
    """
        Find relevant papers similar to an input string


        Arguments:
            input_string: str. String to match against database
            N: int. Number of papers to suggest
            since: int or None. If an int is passed it must be a year,
                only papers more recent than the given year are kept for recomendation
            to: int or None. If an int is passed it must be a year,
                only papers older than that are kept for recomendation
            csv_path: str, Path. Path pointing to a .csv file where the recomendations
                will be saved
            debug: bool. If true refy is set in debug mode and more info are printed
    """
    if debug:
        refy.set_logging("DEBUG")

    refy.query(
        input_string, N=N, since=since, to=to, csv_path=csv_path,
    )


@app.command()
def author(
    author: str = typer.Argument(..., help="Author name"),
    N: int = typer.Option(10, "-N", help="number of recomendations to return"),
    since: int = typer.Option(
        None, "-since", help="Only keep papers published after SINCE"
    ),
    to: int = typer.Option(
        None, "-to", help="Only keep papers published before TO"
    ),
    csv_path: str = typer.Option(
        None, "-csv-path", help="Save suggestions to file"
    ),
    debug: bool = typer.Option(
        False, "-debug", "--d", help="set debug mode ON/OFF"
    ),
):
    """
        Find relevant papers similar to an input string


        Arguments:
            author: str. Author name for search
            N: int. Number of papers to suggest
            since: int or None. If an int is passed it must be a year,
                only papers more recent than the given year are kept for recomendation
            to: int or None. If an int is passed it must be a year,
                only papers older than that are kept for recomendation
            csv_path: str, Path. Path pointing to a .csv file where the recomendations
                will be saved
            debug: bool. If true refy is set in debug mode and more info are printed
    """
    if debug:
        refy.set_logging("DEBUG")

    refy.query_author(
        author, N=N, since=since, to=to, csv_path=csv_path,
    )


@app.command()
def example():
    """
        Run refy on an example .bib file.
    """
    refy.suggest(refy.settings.example_path, N=10)


@app.command()
def update_database(
    folder: str = typer.Argument(
        ..., help="Path to folder with semantic scholar raw data"
    ),
):
    """
        Updates all database files with current settings.

        Arguments:
            folder: str. Path to folder with 
                semantic scholar raw data
    """
    logger.debug("CLI: updating database")
    logger.debug("Unpacking sem. schol. database")
    refy.database.semantic_scholar.upack_database(folder)

    logger.debug("Making sem. schol. database")
    refy.database.semantic_scholar.make_database(folder)

    logger.debug("Making biorxiv database")
    refy.database.biorxiv.make_biorxiv_database()


@app.command()
def train(
    epochs: int = typer.Option(
        50, "--epochs", help="Number of iterations for training"
    ),
    vecs: int = typer.Option(500, "--vecs", help="Size of features vector"),
    lr: float = typer.Option(0.025, "--lr", help="Learning rate"),
):
    """
        Trains the doc2vec model on the database data and abstracts

        Arguments:
            epochs: int. Numberof epochs for training
            vecs: int. Dimensionality of the feature vectors
            lr: float. The initial learning rate
    """
    refy.set_logging("DEBUG")

    logger.debug("CLI: training d2v model")
    refy.doc2vec.train_doc2vec_model(n_epochs=epochs, vec_size=vecs, alpha=lr)


if __name__ == "__main__":
    app()



from rich.progress import (
    Progress,
    BarColumn,
    DownloadColumn,
    TextColumn,
    TransferSpeedColumn,
    TimeRemainingColumn,
)

from myterial import (
    orange,
    amber_light,
)


# ---------------------------------- columns --------------------------------- #


class CurrentTaskColumn(TextColumn):
    _renderable_cache = {}

    def __init__(self, *args):
        pass

    def render(self, task):
        try:
            return f"[{amber_light}]curr. task: [bold {orange}]{task.fields['current_task']}"
        except (AttributeError, TypeError):
            return ""


class FileNameColumn(TextColumn):
    _renderable_cache = {}

    def __init__(self, *args):
        pass

    def render(self, task):
        try:
            return f"[bold {orange}]{task.fields['filename']}"
        except (AttributeError, TypeError):
            return ""


class LossColumn(TextColumn):
    _renderable_cache = {}

    def __init__(self, *args):
        pass

    def render(self, task):
        try:
            return f"[{amber_light}]curr. loss: [bold {orange}]{task.fields['loss']:.3f}"
        except (AttributeError, TypeError):
            return ""


# ------------------------------- progress bars ------------------------------ #
# general porpuse progress
progress = Progress(
    "[progress.description]{task.description}",
    "•",
    TextColumn("[bold magenta]Completed {task.completed}/{task.total}"),
    "•",
    BarColumn(bar_width=None),
    "•",
    TextColumn("Time remaining: ", justify="right"),
    TimeRemainingColumn(),
)

train_progress = Progress(
    "[progress.description]{task.description}",
    "•",
    TextColumn("[bold magenta]Completed {task.completed}/{task.total}"),
    "•",
    LossColumn(),
    "•",
    BarColumn(bar_width=None),
    "•",
    TextColumn("Time remaining: ", justify="right"),
    TimeRemainingColumn(),
)


# Overall progress bar for suggestions
suggest_progress = Progress(
    "[progress.description]{task.description}",
    "•",
    TextColumn("[bold magenta]Completed {task.completed}/{task.total}"),
    "•",
    CurrentTaskColumn(),
    "•",
    BarColumn(bar_width=None),
    "•",
    TextColumn("Time remaining: ", justify="right"),
    TimeRemainingColumn(),
)


http_retrieve_progress = Progress(
    TextColumn("[bold]Downloading: ", justify="right"),
    FileNameColumn(),
    BarColumn(bar_width=None),
    "{task.percentage:>3.1f}%",
    "•",
    DownloadColumn(),
    "• speed:",
    TransferSpeedColumn(),
    "• ETA:",
    TimeRemainingColumn(),
)



from gensim import summarization
import pandas as pd
from loguru import logger

from rich.table import Table
from rich.console import Console
from io import StringIO

from myterial import pink, light_blue_light

# -------------------------------- Highlighter ------------------------------- #
"""
    A highlighter to highlight keywords in paper titles
"""


class Highlighter:
    color = light_blue_light

    def __init__(self, words):
        """
            Highlights pieces of text to mark keywords

            Arguments:
                words: list of str of words to mark
        """
        self.words = words

    def __call__(self, text):
        """
            Highlights a piece of text

            Arguments:
                text: str to highlight

            Returns
                text: highlighted string
        """
        for word in self.words:
            text = text.replace(
                " " + word + " ", f"[{self.color}] {word} [/{self.color}]"
            )
        return text


# -------------------------------- get keyword ------------------------------- #


def get_keywords_from_text(text, N, **kwargs):
    """
        Returns a list of N keywords extracted from a string of text

        Arguments:
            text: str
            N: int, number of keywords.

        Returns:
            keywords: list of str of keywords
    """
    return summarization.keywords(text, words=N, split=True, **kwargs)


# ---------------------------------------------------------------------------- #
#                                   Keywords                                   #
# ---------------------------------------------------------------------------- #


class Keywords:
    keep_n = 8  # number of keywords to keep

    def __init__(self, keywords):
        """
            Represents a list of keywords extracted from text

            Arguments:
                keywords: dict. Dict of keyword:score keywords
        """

        # sort kws and keep only words
        self.kws = pd.Series(keywords).sort_values().index[::-1]

        self._kws = self.kws.copy()
        self.kws = self.kws[: self.keep_n]

    def __len__(self):
        return len(self.kws)

    def __rich_console__(self, *args, **kwargs):
        yield self.to_table()

    def __str__(self):
        buf = StringIO()
        _console = Console(file=buf, force_jupyter=False)
        _console.print(self)

        return buf.getvalue()

    def get_highlighter(self):
        """
            Returns a rich.Console that highlights keywords

            Returns:
                console: rich.Console with highlighter
        """
        return Highlighter(self.kws)

    def to_table(self):
        """
            Returns a rich.Table with a view of the keywords
        """
        logger.debug("keywords -> table")
        # create table
        table = Table(
            show_header=False, show_lines=False, expand=False, box=None,
        )
        table.add_row()

        words = [
            f"[{pink}]    [b]{n+1}.[/b] [u {light_blue_light}]{kw}"
            for n, kw in enumerate(self.kws)
        ]
        table.add_row(*words)

        return table



from rich import print
from rich.console import Console
import sys
from loguru import logger

from rich.spinner import Spinner
from rich.text import Text
from rich.terminal_theme import TerminalTheme
from rich.live import Live

from pyinspect.panels import Report
from myterial import orange, salmon, orange_dark

sys.path.append("./")

from refy import download
from refy.suggestions import Suggestions
from refy.authors import Authors
from refy.keywords import Keywords, get_keywords_from_text


# define a theme for HTML exports
# see: https://github.com/willmcgugan/rich/blob/d9d59c6eda/rich/terminal_theme.py
TERMINAL_THEME = TerminalTheme(
    (30, 30, 30),
    (0, 0, 0),
    [
        (255, 255, 255),
        (128, 0, 0),
        (0, 128, 0),
        (128, 128, 0),
        (0, 0, 128),
        (128, 0, 128),
        (0, 128, 128),
        (192, 192, 192),
    ],
    [
        (128, 128, 128),
        (255, 0, 0),
        (0, 255, 0),
        (255, 255, 0),
        (0, 0, 255),
        (255, 0, 255),
        (0, 255, 255),
        (255, 255, 255),
    ],
)


class SimpleQuery:
    def __init__(self, csv_path=None, html_path=None):
        """
            Base class handling the printing and saving of 
            results from queries and suggest calls. 

            Arguments:
                csv_path: str, Path. Path to a .csv where to save
                    the results
                html_path: str, Path. Path to a .HTML to save formatted
                    results to.
        """
        # get all fils
        download.check_files()

        self.csv_path = csv_path
        self.html_path = html_path
        self.keywords = None

    def __rich_console__(self, *args, **kwargs):
        "Simple query"

    def __str__(self):
        "Simple query"

    def __repr__(self):
        "Simple query"

    def fill(self, papers, N, since, to, ignore_authors=False):
        """
            Given a dataframe of papers and some arguments creates and 
            stores an instance of Suggestions and Authors

            Arguments:
                papers: pd. DataFrame of recomended papers
                N: int. Number of papers to suggest
                since: int or None. If an int is passed it must be a year,
                    only papers more recent than the given year are kept for recomendation
                to: int or None. If an int is passed it must be a year,
                    only papers older than that are kept for recomendation
                ignore_authors: bool. If true the authors information is not extracted
        """
        # create suggestions
        self.suggestions = Suggestions(papers)
        self.suggestions.filter(since=since, to=to)
        self.suggestions.truncate(N)

        # get authors
        if ignore_authors:
            self.authors = []
        else:
            self.authors = Authors(self.suggestions.get_authors())

    def start(self, text):
        """ starts a spinner """
        self.live = Live(
            Spinner("bouncingBall", text=Text(text, style=orange)),
            refresh_per_second=30,
            transient=True,
        )

        self.live.start()

    def stop(self):
        """ stops a spinner """
        self.live.stop()

    def _make_summary(self, text_title=None, text=None, sugg_title=""):
        """
            Creates a summary with some text, suggested papers and authors

            Arguments:
                text_title: str, title for text section
                text: str, text to place in the initial segment of the report
                sugg_title: str, title for the suggestions table

            Returns:
                summary: pyinspect.Report with content
        """
        # try to get an highlighter
        try:
            highlighter = self.keywords.get_highlighter()
        except:
            highlighter = None

        # print summary
        summary = Report(dim=orange)
        summary.width = 160

        # text
        if text is not None:
            if text_title is not None:
                summary.add(text_title)
            summary.add(text)

        # keywords
        if self.keywords is not None:
            summary.add(f"[bold {salmon}]:mag:  [u]keywords\n")
            summary.add(self.keywords.to_table(), "rich")
            summary.spacer()
            summary.line(orange_dark)
            summary.spacer()

        # suggestions
        if sugg_title:
            summary.add(sugg_title)
        summary.add(
            self.suggestions.to_table(highlighter=highlighter), "rich",
        )

        # authors
        if len(self.authors):
            summary.spacer()
            summary.line(orange_dark)
            summary.spacer()

            summary.add(f"[bold {salmon}]:lab_coat:  [u]top authors\n")
            summary.add(self.authors.to_table(), "rich")

        return summary

    def get_keywords(self, papers):
        """
            Extracts set of keywords that best represent the user papers.
            These can be used to improve the search and to improve the
            print out from the query. 

            Arguments:
                papers: pd.DataFrame with papers metadata
        """
        keywords = {}
        for n, (idx, user_paper) in enumerate(papers.iterrows()):
            kwds = get_keywords_from_text(user_paper.abstract, N=10)

            for m, kw in enumerate(kwds):
                if kw in keywords.keys():
                    keywords[kw] += 10 - m
                else:
                    keywords[kw] = 1

        # sort keywords
        self.keywords = Keywords(keywords)

    def print(self, text_title=None, text=None, sugg_title=""):
        """
            Print a summary with some text, suggested papers and authors

            Arguments:
                text_title: str, title for text section
                text: str, text to place in the initial segment of the report
                sugg_title: str, title for the suggestions table
        """
        summary = self._make_summary(
            text_title=text_title, text=text, sugg_title=sugg_title
        )

        print(summary)
        print("")

    def to_html(self, text_title=None, text=None, sugg_title=""):
        """
            Saves the summary view of the query's content to an html file


            Arguments:
                text_title: str, title for text section
                text: str, text to place in the initial segment of the report
                sugg_title: str, title for the suggestions table
        """
        if self.html_path is None:
            return

        logger.debug(f"Saving query to .HTML at: {self.html_path}")
        summary = self._make_summary(
            text_title=text_title, text=text, sugg_title=sugg_title
        )

        console = Console(record=True, width=170)
        console.print(summary)
        console.save_html(self.html_path, theme=TERMINAL_THEME)

    def to_csv(self):
        """
            Saves suggestions to a .csv file
        """
        if self.csv_path is not None:
            self.suggestions.to_csv(self.csv_path)



from refy import Daily
import refy
from pathlib import Path


def test_daily():
    htmlpath = Path("test.html")
    if htmlpath.exists():
        htmlpath.unlink()

    Daily(refy.settings.example_path, html_path=htmlpath)

    assert htmlpath.exists(), "did not save to HTML!"



from refy.suggestions import Suggestions
import pandas as pd
import sys

sys.path.append("./")
from tests import DB
import rich

import pytest


@pytest.fixture
def suggestions():
    sugg = Suggestions(DB)
    sugg.truncate(1000)
    return sugg


def test_suggestions_class_base(suggestions):
    len(suggestions)

    assert isinstance(suggestions.suggestions, pd.DataFrame)

    str(suggestions.truncate(20))
    rich.print(suggestions.truncate(10))

    print(suggestions.titles)


def test_suggestions_class_methods(suggestions):

    suggestions.set_score([1 for i in range(len(suggestions))])

    suggestions.filter(since=2015, to=2019)
    assert suggestions.suggestions.year.min() == 2015
    assert suggestions.suggestions.year.max() == 2019

    suggestions.get_authors()

    suggestions.truncate(10)
    assert len(suggestions) == 10


def test_remove_overlap(suggestions):
    suggestions.remove_overlap(DB)
    assert len(suggestions) == 0, "remove overlap didnt work very well.."



import refy
from typer.testing import CliRunner
from pathlib import Path
import pytest
from refy.cli import app


@pytest.fixture
def csv_path():
    csvpath = Path("test.csv")
    if csvpath.exists():
        csvpath.unlink()
    return str(csvpath)


@pytest.fixture
def html_path():
    htmlpath = Path("test.html")
    if htmlpath.exists():
        htmlpath.unlink()
    return str(htmlpath)


runner = CliRunner()
ex = refy.settings.example_path


def test_cli_daily(html_path):
    runner.invoke(app, ["daily", ex, "-N", "10", "-o", html_path, "--d"])


def test_cli_query(html_path):
    runner.invoke(
        app,
        [
            "query",
            "locomotion",
            "-N",
            "10",
            "-csv-path",
            html_path,
            "--d",
            "-since",
            2015,
            "-to",
            2020,
        ],
    )


def test_cli_author(html_path):
    runner.invoke(
        app,
        [
            "author",
            "Carandini M.",
            "-N",
            "10",
            "-csv-path",
            html_path,
            "--d",
            "-since",
            2015,
            "-to",
            2020,
        ],
    )


def test_cli_suggest(html_path, csv_path):
    runner.invoke(
        app,
        [
            "suggest",
            ex,
            "-N",
            "10",
            "-html-path",
            html_path,
            "-csv-path",
            csv_path,
            "--d",
            "-since",
            2015,
            "-to",
            2020,
        ],
    )



from loguru import logger
import sys

from pyinspect.panels import Report
from myterial import orange, salmon, orange_dark

sys.path.append("./")
from refy.input import load_user_input
from refy.database import load_database
from refy.progress import suggest_progress
from refy import doc2vec as d2v
from refy.suggestions import Suggestions
from refy.authors import Authors
from refy._query import SimpleQuery


class suggest(SimpleQuery):
    suggestions_per_paper = 100  # for each paper find N suggestions

    def __init__(
        self,
        user_papers,
        N=20,
        since=None,
        to=None,
        csv_path=None,
        html_path=None,
    ):
        """
            Suggest new relevant papers based on the user's
            library.

            Arguments:
                user_papers: str, path. Path to a .bib file with user's papers info
                N: int. Number of papers to suggest
                since: int or None. If an int is passed it must be a year,
                    only papers more recent than the given year are kept for recomendation
                to: int or None. If an int is passed it must be a year,
                    only papers older than that are kept for recomendation
                csv_path: str, Path. Path pointing to a .csv file where the recomendations
                    will be saved
                html_path: str, Path. Path to a .html file 
                    where to save the output
        """
        SimpleQuery.__init__(self, csv_path=csv_path, html_path=html_path)

        self.since = since
        self.to = to

        with suggest_progress as progress:
            self.progress = progress
            self.n_completed = -1
            self.task_id = self.progress.add_task(
                "Suggesting papers..", start=True, total=5, current_task="",
            )
            # load data
            self.load_data(user_papers)

            # load d2v model
            self._progress("Loading Doc2Vec model")
            self.d2v = d2v.D2V()

            # get keywords
            self._progress("Extracting keywords from data")
            self.get_keywords(self.user_papers)

            # get suggestions
            self.get_suggestions(N=N)
        self.print()

        # save to .csv and .HTML file
        self.to_csv()
        self.to_html()

    @property
    def n_papers(self):
        return len(self.database)

    @property
    def n_user_papers(self):
        return len(self.user_papers)

    def _progress(self, task_name):
        """
            Update progress bar
        """
        self.n_completed += 1
        self.progress.update(
            self.task_id, current_task=task_name, completed=self.n_completed
        )

    def load_data(self, user_papers):
        """
            Load papers metadata for user papers and database papers

            Arguments:
                user_papers: str, path. Path to a .bib file with user's papers info
        """
        # load database
        self._progress("Loading database papers")
        self.database = load_database()

        # load user data
        self._progress("Loading user papers",)
        self.user_papers = load_user_input(user_papers)

    def suggest_for_paper(self, user_paper_title, user_paper_abstract):
        """
            Finds the best matches for a single paper

            Arguments:
                user_paper_title: str. Title of input user paper
                user_paper_abstract: str. Abstract of input user paper

            Returns:
                suggestions: dict. Dictionary of title:value where value 
                    is self.suggestions_per_paper for the best match paper and 
                    1 for the lowest match
        """
        # find best match with d2v
        best_IDs = self.d2v.predict(
            user_paper_abstract, N=self.suggestions_per_paper
        )

        # get selected papers
        selected = self.database.loc[self.database["id"].isin(best_IDs)]

        if selected.empty:
            logger.debug(
                f'Could not find any suggested papers for paper: "{user_paper_title}" '
            )

        return {
            t: self.suggestions_per_paper - n
            for n, t in enumerate(selected.title.values)
        }

    def _collate_suggestions(self, points):
        """
            Given a dictionart of points for each suggested paper,
            this function returns a dataframe with papers ordered 
            by their score

            Arguments:
                points: dict of title:point entries for each recomended paper

            Returns
                suggestions: Suggestions with suggested papers ordred by score
        """
        # collate recomendations
        suggestions = Suggestions(
            self.database.loc[self.database.title.isin(points.keys())]
        )

        # drop suggestions whose title is in the user papers
        suggestions.remove_overlap(self.user_papers)

        # Get each paper's score
        max_score = self.suggestions_per_paper * self.n_user_papers
        score = [points[title] / max_score for title in suggestions.titles]
        suggestions.set_score(score)

        # keep only papers published within a given years range
        suggestions.filter(to=self.to, since=self.since)

        return suggestions

    def get_suggestions(self, N=20):
        """
            Finds the papers from the database that are not in the user's
            library but are most similar to the users papers.
            For each user paper, get the N most similar papers, then get
            the papers that came up most frequently across all user papers.
            

            Arguments:
                N: int, number of best papers to keep

        """
        logger.debug(f"Getting suggestions for {self.n_user_papers} papers")

        # progress bar
        self._progress("Looking for good papers")
        select_task = self.progress.add_task(
            "Selecting the very best...",
            start=True,
            total=self.n_user_papers,
            current_task="analyzing...",
        )

        # find best matches for each paper
        points = {}
        for n, (idx, user_paper) in enumerate(self.user_papers.iterrows()):
            # keep track of recomendations across all user papers
            paper_suggestions = self.suggest_for_paper(
                user_paper.title, user_paper.abstract
            )

            for suggested, pts in paper_suggestions.items():
                if suggested in points.keys():
                    points[suggested] += pts
                else:
                    points[suggested] = pts

            self.progress.update(select_task, completed=n)
        self.progress.remove_task(select_task)
        self.progress.remove_task(self.task_id)

        # collate and print suggestions
        self.suggestions = self._collate_suggestions(points)
        self.suggestions.get_authors()

        self.suggestions.truncate(N)

    def _make_summary(self, *args, **kwargs):
        """
            Creates a summary with suggestions, authors
            and keywords metadata etc. 

            Arguments:
                *args, **kwargs: not used, for compatibility with
                    SimpleQuery.print's compatibility

            Returns:
                summary: Summary with all information about the results
        """
        # create a list of most recomended authors
        authors = Authors(self.suggestions.authors)

        # get console with highlighter
        highlighter = self.keywords.get_highlighter()

        # create summary
        summary = Report(dim=orange)
        summary.width = 160

        # keywords
        summary.add(f"[bold {salmon}]:mag:  [u]keywords\n")
        summary.add(self.keywords.to_table(), "rich")
        summary.spacer()
        summary.line(orange_dark)
        summary.spacer()

        # suggestions
        summary.add(f"[bold {salmon}]:memo:  [u]recomended paper\n")
        summary.add(self.suggestions.to_table(highlighter=highlighter), "rich")
        summary.spacer()
        summary.line(orange_dark)
        summary.spacer()

        # authors
        summary.add(f"[bold {salmon}]:lab_coat:  [u]top authors\n")
        summary.add(authors.to_table(), "rich")
        summary.spacer()

        return summary


if __name__ == "__main__":
    import refy

    refy.settings.TEST_MODE = False

    refy.set_logging("DEBUG")

    suggest(refy.settings.example_path, N=25, since=2018).print()



from rich.table import Table
from rich.console import Console
from rich.columns import Columns
from io import StringIO
from loguru import logger

from myterial import pink, light_green


class Authors:
    def __init__(self, authors):
        """
            Class to represent a list of authors and 
            fetch metadata about them.

            Arguments:
                authors: list of str of authors names
        """
        self.authors = authors
        logger.debug(f"Spawned Authors with: {len(authors)} authors")

    def __len__(self):
        return len(self.authors)

    def __rich_console__(self, *args, **kwargs):
        yield self.to_table()

    def __str__(self):
        buf = StringIO()
        _console = Console(file=buf, force_jupyter=False)
        _console.print(self)

        return buf.getvalue()

    def to_table(self):
        """
            Returns a rich.Columns object storing tables
            with a view of the authors
        """
        logger.debug("authors -> table")

        def make_table():
            # create table
            table = Table(
                show_header=False, show_lines=False, expand=False, box=None,
            )

            table.add_column(
                style=f"b {pink}", overflow="ellipsis", justify="right"
            )
            table.add_column(style=light_green)
            return table

        if not self.authors:
            return "No authors"

        tables, table = [], None
        for n, author in enumerate(self.authors):
            if not author:
                if table is None:
                    table = make_table()  # otherwise it will give errors
                continue

            # change table evry 4 authors
            if n % 4 == 0:
                if n > 0:
                    tables.append(table)
                if len(tables) == 5:
                    break
                table = make_table()

            table.add_row(f"{n+1}. ", author)

        if len(self) <= 4:
            tables.append(table)

        return Columns(tables, width=28, equal=True, align="left")



from pyinspect import install_traceback

install_traceback(hide_locals=True)

from refy.suggest import suggest
from refy.daily import Daily
from refy.query import query_author, query
from refy.settings import (
    DEBUG,
    base_dir,
)


# ----------------------------- logging settings ----------------------------- #

from loguru import logger
import sys
from pathlib import Path


def set_logging(level="INFO", path=None):
    logger.remove()
    logger.add(sys.stdout, level=level)

    path = path or str(base_dir / "log.log")
    if Path(path).exists():
        Path(path).unlink()

    logger.add(path, level="DEBUG")


if not DEBUG:
    set_logging()
else:
    set_logging(level="DEBUG")



from loguru import logger
from rich import print
import sys

from myterial import orange, salmon, amber_light

sys.path.append("./")
from refy.database import load_database
from refy import doc2vec as d2v
from refy._query import SimpleQuery


class query_author(SimpleQuery):
    def __init__(self, author, N=20, since=None, to=None, csv_path=None):
        """
            Print all authors in the database from a list of authors

            Arguments:
                authors: str with author name
                N: int. Number of papers to suggest
                since: int or None. If an int is passed it must be a year,
                    only papers more recent than the given year are kept for recomendation
                to: int or None. If an int is passed it must be a year,
                    only papers older than that are kept for recomendation
                csv_path: str, Path. Path pointing to a .csv file where the recomendations
                    will be saved
        """

        def clean(string):
            for punc in ".,;)(":
                string = string.replace(punc, "")
            return string

        SimpleQuery.__init__(self, csv_path=csv_path)
        self.start("extracting author's publications")

        logger.debug(f"Fining papers by from author: '{author}'")

        # load and clean database
        papers = load_database()
        papers["clean_authors"] = papers.authors.apply(clean)

        # select papers
        for auth in author.split(" "):
            papers = papers.loc[
                papers.clean_authors.str.contains(
                    " " + clean(auth) + " ", case=False
                )
            ]

        logger.debug(
            f"Found {len(papers)} papers for author's cleaned name: {author}"
        )

        if papers.empty:
            print(
                f"\n[{salmon}]Could not find any papers for author: {author}"
            )
            self.stop()
            return

        # fill
        self.fill(papers, N, since, to)

        # print
        self.stop()
        self.print(
            sugg_title=f'Suggestions for author(s): [bold {orange}]"{author}"\n'
        )

        # save to file
        self.to_csv()


class query(SimpleQuery):
    def __init__(self, input_string, N=20, since=None, to=None, csv_path=None):
        """
            Finds recomendations based on a single input string (with keywords,
            or a paper abstract or whatever) instead of an input .bib file

            Arguments:
                input_stirng: str. String to match against database
                N: int. Number of papers to suggest
                since: int or None. If an int is passed it must be a year,
                    only papers more recent than the given year are kept for recomendation
                to: int or None. If an int is passed it must be a year,
                    only papers older than that are kept for recomendation
                csv_path: str, Path. Path pointing to a .csv file where the recomendations
                    will be saved

            Returns:
                suggestions: pd.DataFrame of N recomended papers
        """
        logger.debug("suggest one")
        SimpleQuery.__init__(self, csv_path=csv_path)
        self.start("Finding recomended papers")

        # load database and abstracts
        database = load_database()

        # load model
        model = d2v.D2V()

        # find recomendations
        best_IDs = model.predict(input_string, N=N)

        # fill
        papers = database.loc[database["id"].isin(best_IDs)]
        if papers.empty:
            print(
                f'\n[bold {salmon}]Could not find any suggested paper with query: "{input_string}"'
            )
            return

        self.fill(papers, N, since, to)

        # print
        self.stop()
        self.print(
            text_title=f"[bold {salmon}]:mag:  [u]search keywords\n",
            text=f"      [b {amber_light}]" + input_string + "\n",
            sugg_title=f"Suggestions:",
        )

        # save to file
        self.to_csv()


if __name__ == "__main__":
    import refy

    refy.settings.TEST_MODE = False

    refy.set_logging("DEBUG")

    # query("locomotion control mouse steering goal directed")
    # query(
    #     "neuron gene expression", N=20, since=2015, to=2018,
    # )

    query_author("Branco Tiago")
    # query_author("Gary Stacey")
    # query_author("Gary  Stacey")
    # query_author("Carandini M.")

    # query_author("carandini")



from setuptools import setup, find_namespace_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

requirements = [
    "pyinspect",
    "gensim",
    "nltk",
    "langdetect",
    "pandas",
    "pyarrow",
    "loguru",
    "requests",
    "myterial",
    "rich",
    "bibtexparser",
    "cython",
    "typer",
    "scholarly",
    "myterial",
    "python-crontab",
]

setup(
    name="refy",
    version="0.3.3",
    description="A scientific papers recomendation tool.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    install_requires=requirements,
    extras_require={
        "dev": ["pytest", "pytest-sugar", "pytest-cov", "coverage"]
    },
    python_requires=">3.6",
    packages=find_namespace_packages(exclude=("tests, examples")),
    entry_points={"console_scripts": ["refy = refy.cli:app"]},
    include_package_data=True,
    url="https://github.com/FedeClaudi/refy",
    author="Federico Claudi",
    zip_safe=False,
)



import json
import requests
import subprocess
import os
from loguru import logger


def isin(l1, l2):
    """
        Checks if any element of a list is included in a second list,
        returns either True or False

        Arguments:
            l1, l2: lists

        Returns:
            is_in: True or False
    """
    if not l1 or not l2:
        return False
    return any(x in l2 for x in l1)


def get_authors(paper):
    """
        Gets the authors of a paper and returns them as a list

        Arguments:
            paper: pd.Series with paper metadata

        Returns:
            authors: list of str of authors
    """
    if paper.source == "biorxiv":
        splitter = "; "
    else:
        splitter = ", "

    return paper.authors.split(splitter)


# --------------------------------- internet --------------------------------- #


def open_in_browser(url):
    """
        Open an url or .html file in default web browser

        Arguments:
            url: str, Path. url or .html file
    """
    url = str(url)

    try:  # should work on Windows
        os.startfile(url)
    except AttributeError:
        try:  # should work on MacOS and most linux versions
            subprocess.call(["open", url])
        except:
            logger.debug("Could not open URL")


def check_internet_connection(
    url="http://www.google.com/", timeout=2, raise_error=True
):
    """Check that there is an internet connection
    url : str
        url to use for testing (Default value = 'http://www.google.com/')
    timeout : int
        timeout to wait for [in seconds] (Default value = 2).
    raise_error : bool
        if false, warning but no error.
    """

    try:
        _ = requests.get(url, timeout=timeout)
        return True
    except requests.ConnectionError:
        if not raise_error:
            print("No internet connection available.")
        else:
            raise ConnectionError(
                "No internet connection, try again when you are connected to the internet."
            )
    return False


def raise_on_no_connection(func):  # pragma: no cover
    """
        Decorator to avoid running a function when there's no internet
    """

    def inner(*args, **kwargs):
        if not check_internet_connection():
            raise ConnectionError("No internet connection found.")
        else:
            return func(*args, **kwargs)

    return inner


def _request(url, stream=False):
    """
        Sends a request to an url and
        makes sure it worked
    """
    response = requests.get(url, stream=stream)
    if not response.ok:
        raise ValueError(
            f"Failed to get a good response when retrieving from {url}. Response: {response.status_code}"
        )
    return response


@raise_on_no_connection
def request(url):
    """ 
        Sends a request to a URL and returns the JSON
        it fetched (if it went through).
    """
    response = _request(url, stream=False)
    return response.json()


# --------------------------------- File I/O --------------------------------- #


def to_json(obj, fpath):
    """ saves an object to json """
    with open(fpath, "w") as out:
        json.dump(obj, out)


def from_json(fpath):
    """ loads an object from json """
    with open(fpath, "r") as fin:
        return json.load(fin)



from rich.table import Table
from rich.console import Console
from io import StringIO
from rich import print
import pandas as pd

from loguru import logger

from myterial import orange, amber, pink, light_green, blue_grey_lighter

from refy.utils import get_authors


class Suggestions:
    def __init__(self, suggestions):
        """
            Class to represent a set of suggested papers and 
            do operations on it (e.g. cleaning/filtering)
        
            Arguments:
                suggestions: pd.DataFrame with suggestions data
        """
        self.suggestions = suggestions.drop_duplicates(subset="title").copy()
        self.suggestions["score"] = None  # set it as None to begin with

    def __len__(self):
        return len(self.suggestions)

    def __rich_console__(self, *args, **kwargs):
        yield self.to_table()

    def __str__(self):
        buf = StringIO()
        _console = Console(file=buf, force_jupyter=False)
        _console.print(self)

        return buf.getvalue()

    @property
    def titles(self):
        return self.suggestions.title.values

    def _reset_idx(self):
        self.suggestions = self.suggestions.reset_index(drop=True)

    def to_csv(self, filepath):
        """
            Save dataframe to file

            Arguments:
                filepath: str, Path to .csv file
        """
        self.suggestions.to_csv(filepath)

    def truncate(self, N):
        """
            Keep first N suggestions

            Arguments:
                N: int.
        """
        self.suggestions = self.suggestions[:N]

    def set_score(self, score):
        """
            Fills in the score column with given values and sorts
            the suggestions according to the score

            Arguments:
                score: list of float with score for each paper
        """
        self.suggestions["score"] = score
        self.suggestions = self.suggestions.sort_values(
            "score", ascending=False
        )
        self._reset_idx()

    def remove_overlap(self, user_papers):
        """
            Remove suggestions that appear to overlap with input user papers

            Arguments:
                user_papers: pd.DataFrame with user paper metadata
        """
        self.suggestions = self.suggestions.loc[
            ~self.suggestions.title.isin(user_papers.title)
        ]

        # self.suggestions = self.suggestions.loc[
        #     ~self.suggestions.doi.isin(user_papers.doi)
        # ]

        self._reset_idx()

    def filter(self, since=None, to=None):
        """
            Keep a subset of suggested papers matching criteria

            Arguments:
                since: int or None. If an int is passed it must be a year,
                    only papers more recent than the given year are kept for recomendation
                to: int or None. If an int is passed it must be a year,
                    only papers older than that are kept for recomendation
        """

        if since:
            self.suggestions = self.suggestions.loc[
                self.suggestions.year >= int(since)
            ]
        if to:
            self.suggestions = self.suggestions.loc[
                self.suggestions.year <= int(to)
            ]
        self._reset_idx()

    def get_authors(self):
        """
            Gets the authors that appear in the recomended papers
            sorted by how frequently they appear.

            Returns:
                authors: list of str of authors names
        """
        # Get how often authors show up
        authors = {}
        for i, paper in self.suggestions.iterrows():
            for author in get_authors(paper):
                if author in authors.keys():
                    authors[author] += 1
                else:
                    authors[author] = 1

        # sort authors
        authors = pd.Series(authors)
        if authors.empty:
            self.authors = []
        else:
            self.authors = list(pd.Series(authors).sort_values().index)[::-1]
        return self.authors

    def to_table(self, title=None, highlighter=None):
        """
            Creates a Rich table/panel with a nice representation of the papers

            Arguments:
                title: str. Optional input to replace default title
                highlighter: Highlighter for mark keywords

        """
        logger.debug("Suggestions -> table")
        if self.suggestions.empty:
            print(f"[{orange}]Found no papers matching your query, sorry")
            return "no suggestions found"

        # create table
        table = Table(
            show_header=True,
            header_style=f"bold {pink}",
            show_lines=True,
            expand=False,
            box=None,
            caption=f"[{blue_grey_lighter}]{len(self.suggestions)} papers, recommended by refy :ok_hand:",
            caption_style="dim",
            padding=(0, 1),
        )
        table.add_column("#", style=pink)
        table.add_column("score", style="dim", justify="left")
        table.add_column("year", style="dim", justify="center")
        table.add_column(
            "author",
            style=f"{light_green}",
            justify="right",
            width=22,
            overflow="crop",
        )
        table.add_column(
            "title",
            style=f"bold {orange}",
            max_width=80,
            justify="left",
            overflow="fold",
        )
        table.add_column("DOI", style="dim")

        # add papers to table
        for i, paper in self.suggestions.iterrows():
            if paper.score:
                score = f"[{blue_grey_lighter} dim]" + str(
                    round(paper["score"], 3)
                )
            else:
                score = ""

            if paper.doi:
                url = f"[{blue_grey_lighter}][link=https://doi.org/{paper.doi}]{paper.doi}[/]"
            else:
                url = f"[{blue_grey_lighter}][link={paper.url}]link[/]"

            authors = get_authors(paper)
            if len(authors) > 1:
                author = authors[0] + "[dim] et al."
            else:
                author = authors[0]

            if highlighter is None:
                title = paper.title
            else:
                title = highlighter(paper.title)

            table.add_row(
                str(i + 1),
                score,
                f"[dim {amber}]" + str(paper.year),
                author,
                title,
                url,
            )

        # fit in a panel
        return table
